from ASTVisitor import ASTVisitor
import AST

#Info: Visitor constructing a possible string representation
#      of an AST.
class StrReprVisitor(ASTVisitor):

    def __init__(self):
    #Info: initialize the (private) string representation to 
    #      be the empty string
        self.__strRepr=""

    def setStrRepr(self, strRepr=""):
    #Info: set the (private) string representation
        self.__strRepr = strRepr

    def getStrRepr(self):
    #Info: return the (private) string representation
        return self.__strRepr

    #Info: The str repr of a number is simply its value
    def __numberStr(self,number):
        return str(number.getValue())

    #Info: The str repr of a CellRef is something like A5, $A5, $A$5,
    #      depending if the reference is absolute or relative, with respect
    #      to the row/column
    def __cellRefStr(self,cellRef):
        absCol=""
        absRow=""
        if cellRef.isAbsColumn():
            absCol="$"
        if cellRef.isAbsRow():
            absRow="$"
        return absCol+AST.intToString(cellRef.getColumn())+absRow+str(cellRef.getRow())

    #Info: The str repr of a number is simply its name
    def __functionStr(self,function):
        return function.getName()

    def visitNumber(self, number):
        self.__strRepr+=self.__numberStr(number)

    def visitCellRef(self, cellRef):
        self.__strRepr+=self.__cellRefStr(cellRef)

    #Info: Visit a Function, construct the __strRepr,
    #      e.g. sum(4,3) (thus, calling the accept method
    #      of the children (i.e: arguments) of the function.
    def visitFunction(self, function):
        self.__strRepr+=self.__functionStr(function)+"("
        args=function.getArgs()
        for i in xrange(len(args)):
            if i!=0:
                self.__strRepr+=","
            args[i].accept(self)
        self.__strRepr+=")"

    #Info: Constructing the RangeRef __strRepr by
    #      visiting its two CellRef (adding ":"
    #      in-between)
    def visitRangeRef(self, rangeRef):
        #Visit first corner: (CellRef node)
        rangeRef.getFirstCorner().accept(self)
        self.__strRepr+=":"
        #Visit second corner: (CellRef node)
        rangeRef.getSecondCorner().accept(self)

